%% Calculate various estimates and true values
% Contact velocity
y2      = ss_mat.C2 * x;                            % True contact velocity
y2c     = ss_mat.C2 * xc(1:(2*ss_mat.n_dof), :);    % Corrected contact velocity estimate

% Corrected contact force estimate
fc_est  = zeros(ss_mat.n_u_in, ss_mat.N);       % Initialize vector
for i   = 1:ss_mat.N
    fc_est(:,i)	= fcontact(y2c(:, i), xc(7, i), xc(8, i));
end

% True contact force
fc_true = fcontact(y2, alpha, beta);

% Innovation of prediction
innov   = y - yp;

% Correction term (theoretically equal to xc - xp)
x_correction	= zeros(ss_mat.n_state, ss_mat.N);
for i   = 1:ss_mat.N
    x_correction(:, i)  = K_t(:, :, i)*innov(:, i);
end

% Corrected output error
err     = y - yc;

% Power dissipated by contact
Pdiss_cont_est      = sum( abs(fc_est.*y2c), 1) ;    % Estimated power dissipation from contact
Pdiss_cont_true     = sum( abs(fc_true.*y2), 1) ;    % True power dissipation from contact

% Power dissipated by the core barrel structure
load('c_damp.mat');
Pdiss_struc_est     = zeros(1, ss_mat.N);
Pdiss_struc_true    = zeros(1, ss_mat.N);
for i = 1:ss_mat.N
    Pdiss_struc_est     = sum( 0.5*xc(4:6, i)'*C_damp*xc(4:6, i), 1 );  % Estimated structural power dissipation
    Pdiss_struc_true    = sum( 0.5*x(4:6, i)'*C_damp*x(4:6, i), 1 );    % True structural power dissipation
end

% Total power dissipated
Pdiss_est 	= Pdiss_cont_est + Pdiss_struc_est;
Pdiss_true  = Pdiss_cont_true + Pdiss_struc_true;

% Estimated contact force function slope
gamma_est   = xc(7, :) .* xc(8, :);
gamma_true  = alpha * beta;

%% Plot some stuff
% Measured time response
figure(1),
subplot(4,1,1)
plot(time, y(1, :), time, yp(1, :), time, yc(1, :));
title('System Outputs')
% xlim([0, time(ss_mat.N)])
xlim([0, 10]);
xlabel('Time');
ylabel('Measurement 1');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(4,1,2)
plot(time, y(2, :), time, yp(2, :), time, yc(2, :));
% xlim([0, time(ss_mat.N)])
xlim([0, 10]);
xlabel('Time');
ylabel('Measurement 2');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(4,1,3)
plot(time, y(3, :), time, yp(3, :), time, yc(3, :));
% xlim([0, time(ss_mat.N)])
xlim([0, 10]);
xlabel('Time');
ylabel('Measurement 3');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(4,1,4)
plot(time, y(4, :), time, yp(4, :), time, yc(4, :));
% xlim([0, time(ss_mat.N)])
xlim([0, 10]);
xlabel('Time');
ylabel('Measurement 4');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');

% Position state estimates
figure(2),
subplot(3,1,1)
plot(time, x(1,:), ...
    time, xp(1,:), ...
    time, xc(1,:));
% xlim([0, time(ss_mat.N)])
xlim([0, 10])
xlabel('Time');
ylabel('Horizontal Position');
title('Position State Estimates')
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(3,1,2)
plot(time, x(2,:), ...
    time, xp(2,:), ...
    time, xc(2,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Vertical Position');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(3,1,3)
plot(time, x(3,:), ...
    time, xp(3,:), ...
    time, xc(3,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Rotational Position');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');

% Velocity state estimates
figure(3),
subplot(3,1,1)
plot(time, x(4,:), ...
    time, xp(4,:), ...
    time, xc(4,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Horizontal Velocity');
title('Velocity State Estimates')
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(3,1,2)
plot(time, x(5,:), ...
    time, xp(5,:), ...
    time, xc(5,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Vertical Velocity');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');
subplot(3,1,3)
plot(time, x(6,:), ...
    time, xp(6,:), ...
    time, xc(6,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Rotational Velocity');
legend('True', 'Predicted', 'Corrected', ...
    'Location', 'SouthEast');

% Estimated parameters
figure(4)
subplot(3,1,1)
plot(time, alpha*ones(1, ss_mat.N), time, xc(7,:), time, alpha_avg*ones(1, ss_mat.N));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Alpha');
title('Parameter Estimates')
legend('True', 'Estimate', 'Average')
subplot(3,1,2)
plot(time, beta*ones(1, ss_mat.N), time, xc(8,:), time, beta_avg*ones(1, ss_mat.N));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Beta');
legend('True', 'Estimate', 'Average')
subplot(3,1,3)
plot(time, gamma_true*ones(1, ss_mat.N), time, gamma_est, time, alpha_avg*beta_avg*ones(1, ss_mat.N));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Gamma');
legend('True', 'Estimate', 'Average')

% Contact velocity
figure(5)
subplot(4,1,1)
plot(time, y2(1,:), time, y2c(1,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact velocity 1 (y)');
title('Contact point velocity over time')
legend('True', 'Estimate')
subplot(4,1,2)
plot(time, y2(2,:), time, y2c(2,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact velocity 2 (y)');
legend('True', 'Estimate')
subplot(4,1,3)
plot(time, y2(3,:), time, y2c(3,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact velocity 1 (z)');
legend('True', 'Estimate')
subplot(4,1,4)
plot(time, y2(4,:), time, y2c(4,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact velocity 2 (z)');
legend('True', 'Estimate')

% Contact force
figure(6)
subplot(4,1,1)
plot(time, fc_true(1,:), time, fc_est(1,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact force 1 (y)')
title('Contact force over time')
legend('True', 'Estimate')
subplot(4,1,2)
plot(time, fc_true(2,:), time, fc_est(2,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact force 2 (y)')
legend('True', 'Estimate')
subplot(4,1,3)
plot(time, fc_true(3,:), time, fc_est(3,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact force 1 (z)')
legend('True', 'Estimate')
subplot(4,1,4)
plot(time, fc_true(4,:), time, fc_est(4,:));
xlim([0, time(ss_mat.N)])
xlabel('Time');
ylabel('Contact force 2 (z)')
legend('True', 'Estimate')

% Power Dissipation
figure(7)
subplot(2,1,1)
semilogy(time, Pdiss_true, time, Pdiss_est)
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('Power loss')
title('Total Power Dissipation')
legend('True', 'Estimate')
subplot(2,1,2)
semilogy(time, Pdiss_cont_true, time, Pdiss_cont_est)
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('Contact power loss')
title('Contact Power Dissipation')
legend('True', 'Estimate')

% Contact force model
figure(8)
subplot(4,1,1)
plot(y2(1,:), fc_true(1,:), '.', y2c(1,:), fc_est(1,:), '.');
xlabel('Contact velocity 1 (y)')
ylabel('Contact force 1 (y)')
legend('True', 'Estimate')
title('Contact force model')
subplot(4,1,2)
plot(y2(2,:), fc_true(2,:), '.', y2c(2,:), fc_est(2,:), '.');
xlabel('Contact velocity 2 (y)')
ylabel('Contact force 2 (y)')
legend('True', 'Estimate')
subplot(4,1,3)
plot(y2(3,:), fc_true(3,:), '.', y2c(3,:), fc_est(3,:), '.');
xlabel('Contact velocity 1 (z)')
ylabel('Contact force 1 (z)')
legend('True', 'Estimate')
subplot(4,1,4)
plot(y2(4,:), fc_true(4,:), '.', y2c(4,:), fc_est(4,:), '.');
xlabel('Contact velocity 2 (z)')
ylabel('Contact force 2 (z)')
legend('True', 'Estimate')

% Innovation
figure(9)
subplot(4,1,1)
plot(time, innov(1, :));
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('Innovation 1')
title('Innovation of Output Estimates')
subplot(4,1,2)
plot(time, innov(2, :));
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('Innovation 2')
subplot(4,1,3)
plot(time, innov(3, :));
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('Innovation 3')
subplot(4,1,4)
plot(time, innov(4, :));
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('Innovation 4')

% Correction of positions
figure(10)
subplot(3,1,1)
plot(time, x_correction(1, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('y correction')
title('Correction of position states (K*innov)')
subplot(3,1,2)
plot(time, x_correction(2, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('z correction')
subplot(3,1,3)
plot(time, x_correction(3, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('theta correction')

% Correction of velocities
figure(11)
subplot(3,1,1)
plot(time, x_correction(4, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('ydot correction')
title('Correction of velocity states (K*innov)')
subplot(3,1,2)
plot(time, x_correction(5, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('zdot correction')
subplot(3,1,3)
plot(time, x_correction(5, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('thetadot correction')

% Correction of parameters
figure(12)
subplot(2,1,1)
plot(time, x_correction(7, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('alpha correction')
title('Correction of parameters (K*innov)')
subplot(2,1,2)
plot(time, x_correction(8, :))
xlim([0, time(ss_mat.N)])
xlabel('Time')
ylabel('beta correction')

% Output error
figure(13)
subplot(4,1,1)
plot(time, err(1, :))
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time')
ylabel('Output 1 Error')
title('Error of the Corrected Output Estimates')
subplot(4,1,2)
plot(time, err(2, :))
% xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time')
ylabel('Output 2 Error')
subplot(4,1,3)
plot(time, err(3,:))
% xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time')
ylabel('Output 3 Error')
subplot(4,1,4)
plot(time, err(4,:))
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time')
ylabel('Output 4 Error')

% Position State error
figure(14),
subplot(3,1,1)
plot(time, xc(1,:) - x(1,:));
% xlim([0, time(ss_mat.N)])
xlim([0, 10])
xlabel('Time');
ylabel('Horizontal Position Error');
title('Error of the Corrected Position State Estimates')
subplot(3,1,2)
plot(time, xc(2,:) - x(2,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Vertical Position Error');
subplot(3,1,3)
plot(time, xc(3,:) - x(3,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Rotational Position Error');

% Velocity State error
figure(15),
subplot(3,1,1)
plot(time, xc(4,:) - x(4,:));
% xlim([0, time(ss_mat.N)])
xlim([0, 10])
xlabel('Time');
ylabel('Horizontal Velocity Error');
title('Error of the Corrected Velocity State Estimates')
subplot(3,1,2)
plot(time, xc(5,:) - x(5,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Vertical Velocity Error');
subplot(3,1,3)
plot(time, xc(6,:) - x(6,:));
%xlim([0, time(ss_mat.N)])
xlim([0,10])
xlabel('Time');
ylabel('Rotational Velocity Error');
