% 7/02/2022
% Function for extracting resonance peak information from a discrete-time
% signal in the time-domain
% Nicholas Harn
function [freq, Pyy_dB, wr, Mpk, dw, prom, dr] = extract_peak_info(y, ...
    h, minprom, minwidth_coef, minheight)

% PSD of Signal
[Pyy, freq] = pwelch(y, [], [], [], 1/h);
Pyy_dB = 20*log10(Pyy);         % dB conversion
df = freq(2) - freq(1);         % Frequency resolution
minwidth = minwidth_coef * df;

% Locate resonance peaks in signal
[Mpk, wr, dw, prom] = findpeaks(Pyy_dB, ...
    freq, 'MinPeakProminence', minprom, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', minwidth, 'MinPeakHeight', minheight);

% Damping rato estimate
dr = zeros(length(Mpk), 1);
for i = 1:length(Mpk)
    dr(i) = dw(i) / (2*wr(i));
end

end