% 07/20/2022
% Nicholas Harn
% Comparing damping without contact and damping with contact
clear;
close all;
clc;
rng('shuffle')

%% Loading data without contact
load('data_core_nocontact.mat');      % Load synthetic data
load('model_core_nocontact.mat');     % Load core barrel model
% load('c_damp.mat');

x_nocontact = x;
y_nocontact = y;

%% Damping ratios w/o contact
% PSD of Response 1
[Py1_nocontact, freq_nocontact] = pwelch(y_nocontact(1, :), [], [], [], 1/ss_mat.h);
Py1_dB_nocontact = 20*log10(Py1_nocontact);             % dB conversion
df_nocontact = freq_nocontact(2) - freq_nocontact(1);   % Frequency resolution

% Locate resonance peaks in response
[Mpk1_nocontact, wr1_nocontact, dw1_nocontact, prom1_nocontact] = findpeaks(Py1_dB_nocontact, ...
    freq_nocontact, 'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_nocontact, 'MinPeakHeight', -50);
Y1_nocontact = fft(y_nocontact(1,:));

% Damping rato estimate
dr_y1_nocontact = zeros(length(Mpk1_nocontact), 1);
for i = 1:length(Mpk1_nocontact)
    dr_y1_nocontact(i) = dw1_nocontact(i) / (2*wr1_nocontact(i));
end

% PSD of Response 2
[Py2_nocontact] = pwelch(y_nocontact(2, :), [], [], [], 1/ss_mat.h);
Py2_dB_nocontact = 20*log10(Py2_nocontact);

% Locate Resonance Peaks of Response 2
[Mpk2_nocontact, wr2_nocontact, dw2_nocontact, prom2_nocontact] = findpeaks(Py2_dB_nocontact, ...
    freq_nocontact, 'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_nocontact, 'MinPeakHeight', -50);
Y2_nocontact = fft(y_nocontact(2,:));

% Damping ratio estimates
dr_y2_nocontact = zeros(length(Mpk2_nocontact), 1);
for i = 1:length(Mpk2_nocontact)
    dr_y2_nocontact(i) = dw2_nocontact(i) / (2*wr2_nocontact(i));
end

% PSD of Response 3
[Py3_nocontact] = pwelch(y_nocontact(3, :), [], [], [], 1/ss_mat.h);
Py3_dB_nocontact = 20*log10(Py3_nocontact);

% Locate Resonance Peaks
[Mpk3_nocontact, wr3_nocontact, dw3_nocontact, prom3_nocontact] = findpeaks(Py3_dB_nocontact, ...
    freq_nocontact, 'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_nocontact, 'MinPeakHeight', -50);
Y3_nocontact = fft(y_nocontact(3,:));

% Damping ratio
dr_y3_nocontact = zeros(length(Mpk3_nocontact), 1);
for i = 1:length(Mpk3_nocontact)
    dr_y3_nocontact(i) = dw3_nocontact(i) / (2*wr3_nocontact(i));
end

% PSD of Response 4
[Py4_nocontact] = pwelch(y_nocontact(4, :), [], [], [], 1/ss_mat.h);
Py4_dB_nocontact = 20*log10(Py4_nocontact);

% Locate Resonance Peaks
[Mpk4_nocontact, wr4_nocontact, dw4_nocontact, prom4_nocontact] = findpeaks(Py4_dB_nocontact, ...
    freq_nocontact, 'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_nocontact, 'MinPeakHeight', -50);
Y4_nocontact = fft(y_nocontact(4,:));

% Damping ratio
dr_y4_nocontact = zeros(length(Mpk4_nocontact), 1);
for i = 1:length(Mpk4_nocontact)
    dr_y4_nocontact(i) = dw4_nocontact(i) / (2*wr4_nocontact(i));
end

%% Load synthetic data with contact (v5 is overdamped, v4 is near undamped)
load('data_core_highcontact.mat');
x_true = x;
y_true = y;

%% Damping ratios w/ contact
% PSD of Response 1
[Py1_true, freq_true] = pwelch(y_true(1, :), [], [], [], 1/ss_mat.h);
Py1_dB_true = 20*log10(Py1_true);
df_true = freq_true(2) - freq_true(1);

% Locate Resonance Peaks
[Mpk1_true, wr1_true, dw1_true, prom1_true] = findpeaks(Py1_dB_true, freq_true, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_true, 'MinPeakHeight', -50);
Y1_true = fft(y_true(1,:));

% Damping ratio
dr_y1_true = zeros(length(Mpk1_true), 1);
for i=1:length(Mpk1_true)
    dr_y1_true(i) = dw1_true(i)/(2*wr1_true(i));
end

% PSD of Response 2
[Py2_true] = pwelch(y_true(2, :), [], [], [], 1/ss_mat.h);
Py2_dB_true = 20*log10(Py2_true);

% Locate Resonance Peaks
[Mpk2_true, wr2_true, dw2_true, prom2_true] = findpeaks(Py2_dB_true, freq_true, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_true, 'MinPeakHeight', -50);
Y2_true = fft(y_true(2,:));

% Damping ratio
dr_y2_true = zeros(length(Mpk2_true), 1);
for i=1:length(Mpk2_true)
    dr_y2_true(i) = dw2_true(i)/(2*wr2_true(i));
end

% PSD of Response 3
[Py3_true] = pwelch(y_true(3, :), [], [], [], 1/ss_mat.h);
Py3_dB_true = 20*log10(Py3_true);

% Locate Resonance Peaks
[Mpk3_true, wr3_true, dw3_true, prom3_true] = findpeaks(Py3_dB_true, freq_true, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_true, 'MinPeakHeight', -50);
Y3_true = fft(y_true(3,:));

% Damping ratio
dr_y3_true = zeros(length(Mpk3_true), 1);
for i=1:length(Mpk3_true)
    dr_y3_true(i) = dw3_true(i)/(2*wr3_true(i));
end

% PSD of Output 4
[Py4_true] = pwelch(y_true(4, :), [], [], [], 1/ss_mat.h);
Py4_dB_true = 20*log10(Py4_true);

% Locate Resonance Peaks
[Mpk4_true, wr4_true, dw4_true, prom4_true] = findpeaks(Py4_dB_true, freq_true, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_true, 'MinPeakHeight', -50);
Y4_true = fft(y_true(4,:));

% Damping ratio
dr_y4_true = zeros(length(Mpk4_true), 1);
for i=1:length(Mpk4_true)
    dr_y4_true(i) = dw4_true(i)/(2*wr4_true(i));
end

%% Load UKF estimates w/ contact
load('ukf_estimates.mat');
x_est = xc_states;
y_est = yc;

%% Damping ratio estimates for UKF
% PSD of Output 1
[Py1_est, freq_est] = pwelch(y_est(1, :), [], [], [], 1/ss_mat.h);
Py1_dB_est = 20*log10(Py1_est);
df_est = freq_est(2) - freq_est(1);

% Locate Resonance Peaks
[Mpk1_est, wr1_est, dw1_est, prom1_est] = findpeaks(Py1_dB_est, freq_est, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_est, 'MinPeakHeight', -50);
Y1_est = fft(y_est(1,:));

% Damping ratio
dr_y1_est = zeros(length(Mpk1_est), 1);
for i=1:length(Mpk1_est)
    dr_y1_est(i) = dw1_est(i)/(2*wr1_est(i));
end

% PSD of Output 2
[Py2_est] = pwelch(y_est(2, :), [], [], [], 1/ss_mat.h);
Py2_dB_est = 20*log10(Py2_est);

% Locate Resonance Peaks
[Mpk2_est, wr2_est, dw2_est, prom2_est] = findpeaks(Py2_dB_est, freq_est, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_est, 'MinPeakHeight', -50);
Y2_est = fft(y_est(1,:));

% Damping ratio
dr_y2_est = zeros(length(Mpk2_est), 1);
for i=1:length(Mpk2_est)
    dr_y2_est(i) = dw2_est(i)/(2*wr2_est(i));
end

% PSD of Output 3
[Py3_est] = pwelch(y_est(3, :), [], [], [], 1/ss_mat.h);
Py3_dB_est = 20*log10(Py3_est);

% Locate Resonance Peaks
[Mpk3_est, wr3_est, dw3_est, prom3_est] = findpeaks(Py3_dB_est, freq_est, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_est, 'MinPeakHeight', -50);
Y3_est = fft(y_est(1,:));

% Damping ratio
dr_y3_est = zeros(length(Mpk3_est), 1);
for i=1:length(Mpk3_est)
    dr_y3_est(i) = dw3_est(i)/(2*wr3_est(i));
end

% PSD of Output 4
[Py4_est] = pwelch(y_est(4, :), [], [], [], 1/ss_mat.h);
Py4_dB_est = 20*log10(Py4_est);

% Locate Resonance Peaks
[Mpk4_est, wr4_est, dw4_est, prom4_est] = findpeaks(Py4_dB_est, freq_est, ...
    'MinPeakProminence', 15, 'WidthReference', 'halfprom', ...
    'MinPeakWidth', 15*df_est, 'MinPeakHeight', -50);
Y4_est = fft(y_est(1,:));

% Damping ratio
dr_y4_est = zeros(length(Mpk4_est), 1);
for i=1:length(Mpk4_est)
    dr_y4_est(i) = dw4_est(i)/(2*wr4_est(i));
end

%% Plots

% Response 1
figure(1)
subplot(4,1,1)
semilogx(freq_nocontact, Py1_dB_nocontact, freq_true, Py1_dB_true, ...
    wr1_nocontact, Mpk1_nocontact, 'v', wr1_true, Mpk1_true, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 1, With and Without Contact')
legend('True Response w/o Contact', 'True Response w/ Contact', ...
    'True Peaks w/o Contact', 'True Peaks w/ Contact')

% Response 2
% figure(2)
subplot(4,1,2)
semilogx(freq_nocontact, Py2_dB_nocontact, freq_true, Py2_dB_true, ...
    wr2_nocontact, Mpk2_nocontact, 'v', wr2_true, Mpk2_true, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 2, With and Without Contact')
legend('True Response w/o Contact', 'True Response w/ Contact', ...
    'True Peaks w/o Contact', 'True Peaks w/ Contact')

% Response 3
% figure(3)
subplot(4,1,3)
semilogx(freq_nocontact, Py3_dB_nocontact, freq_true, Py3_dB_true, ...
    wr3_nocontact, Mpk3_nocontact, 'v', wr3_true, Mpk3_true, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 3, With and Without Contact')
legend('True Response w/o Contact', 'True Response w/ Contact', ...
    'True Peaks w/o Contact', 'True Peaks w/ Contact')

% Response 4
% figure(4)
subplot(4,1,4)
semilogx(freq_nocontact, Py4_dB_nocontact, freq_true, Py4_dB_true, ...
    wr4_nocontact, Mpk4_nocontact, 'v', wr4_true, Mpk4_true, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 4, With and Without Contact')
legend('True Response w/o Contact', 'True Response w/ Contact', ...
    'True Peaks w/o Contact', 'True Peaks w/ Contact')

% Comparing PSDs of Estimated vs True Responses w/ Contact
figure(5)
subplot(4,1,1)
semilogx(freq_true, Py1_dB_true, freq_est, Py1_dB_est, ...
    wr1_true, Mpk1_true, 'v', wr1_est, Mpk1_est, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 1')
legend('True Response w/ Contact', 'Estimated Response w/ Contact', ...
    'True Peaks w/ Contact', 'Estimated Peaks w/ Contact')

% figure(6)
subplot(4,1,2)
semilogx(freq_true, Py2_dB_true, freq_est, Py2_dB_est, ...
    wr2_true, Mpk2_true, 'v', wr2_est, Mpk2_est, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 2')
legend('True Response w/ Contact', 'Estimated Response w/ Contact', ...
    'True Peaks w/ Contact', 'Estimated Peaks w/ Contact')

% figure(7)
subplot(4,1,3)
semilogx(freq_true, Py3_dB_true, freq_est, Py3_dB_est, ...
    wr3_true, Mpk3_true, 'v', wr3_est, Mpk3_est, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 3')
legend('True Response w/ Contact', 'Estimated Response w/ Contact', ...
    'True Peaks w/ Contact', 'Estimated Peaks w/ Contact')

% figure(8)
subplot(4,1,4)
semilogx(freq_true, Py4_dB_true, freq_est, Py4_dB_est, ...
    wr4_true, Mpk4_true, 'v', wr4_est, Mpk4_est, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Output 4')
legend('True Response w/ Contact', 'Estimated Response w/ Contact', ...
    'True Peaks w/ Contact', 'Estimated Peaks w/ Contact')

% Condition
figure(9)
semilogx(freq_nocontact, Py1_dB_nocontact, freq_true, Py1_dB_true, ...
    wr1_nocontact, Mpk1_nocontact, 'v', wr1_true, Mpk1_true, 'v')
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of the Core Barrel Response')
legend('True Response w/o Contact (poor condition)', 'True Response w/ Contact (good condition)', ...
    'True Peaks w/o Contact (poor condition)', 'True Peaks w/ Contact (good condition)')

% PSD
figure(10)
semilogx(freq_nocontact, Py1_dB_nocontact)
xlim([0.1 1000])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
title('Power Spectral Density of Core Barrel Output 1')

%% Save damping ratio estimates
% True w/o contact
wr_nocontact =  [wr1_nocontact; wr2_nocontact; wr3_nocontact; wr4_nocontact];
dr_nocontact =  [dr_y1_nocontact; dr_y2_nocontact; dr_y3_nocontact; dr_y4_nocontact];
damp_nocontact =    [0,                 1; ...
                    wr_nocontact,       dr_nocontact];

%writematrix(damp_nocontact, 'Synth_Output_Damping_True_Contactless.csv')

% Estimate w/ contact
wr_est =        [wr1_est; wr2_est; wr3_est; wr4_est];
dr_est =        [dr_y1_est; dr_y2_est; dr_y3_est; dr_y4_est];
damp_est =          [0,      1; ...
                    wr_est, dr_est];

%writematrix(damp_est, 'Synth_Output_Damping_Estimate_Contact.csv');
